/*=========================================================================
 *
 *  Copyright NumFOCUS
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *=========================================================================*/
#ifndef sitkShow_h
#define sitkShow_h

#include "sitkImage.h"
#include "sitkIO.h"
#include "sitkProcessObject.h"

namespace itk::simple
{

/** Display an image in an external viewer (Fiji by default)
 *
 * This function directly calls the execute method of ImageViewer in order to
 * support a procedural API
 *
 **/
void SITKIO_EXPORT
Show(const Image & image, const std::string & title = "", const bool debugOn = ProcessObject::GetGlobalDefaultDebug());
} // namespace itk::simple

#endif
