/*=========================================================================
*
*  Copyright NumFOCUS
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*         http://www.apache.org/licenses/LICENSE-2.0.txt
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*=========================================================================*/
#ifndef sitkBinaryMagnitudeImageFilter_h
#define sitkBinaryMagnitudeImageFilter_h

/*
 * WARNING: DO NOT EDIT THIS FILE!
 * THIS FILE IS AUTOMATICALLY GENERATED BY THE SIMPLEITK BUILD PROCESS.
 * Please look at sitkImageFilterTemplate.h.in to make changes.
 */

#include <memory>

#include "sitkBasicFilters.h"
#include "sitkImageFilter.h"

namespace itk::simple {

    /**\class BinaryMagnitudeImageFilter
\brief Computes the square root of the sum of squares of corresponding input pixels.

This filter is templated over the types of the two input images and the type of the output image.

Numeric conversions (castings) are done by the C++ defaults.

The filter walks over all of the pixels in the two input images, and for each pixel does the following:



\li cast the input 1 pixel value to double 


\li cast the input 2 pixel value to double 


\li compute the sum of squares of the two pixel values 


\li compute the square root of the sum 


\li cast the double value resulting from std::sqrt() to the pixel type of the output image 


\li store the cast value into the output image.



The filter expects all images to have the same dimension (e.g. all 2D, or all 3D, or all ND)
\sa itk::simple::BinaryMagnitude for the procedural interface
\sa itk::BinaryMagnitudeImageFilter for the Doxygen on the original ITK class.
     */
    class SITKBasicFilters_EXPORT BinaryMagnitudeImageFilter : public ImageFilter {
    public:
      using Self = BinaryMagnitudeImageFilter;

      /** Destructor */
      virtual ~BinaryMagnitudeImageFilter();

      /** Default Constructor that takes no arguments and initializes
       * default parameters */
      BinaryMagnitudeImageFilter();

      /** Define the pixels types supported by this filter */
      using PixelIDTypeList = BasicPixelIDTypeList;


      /** Name of this class */
      std::string GetName() const { return std::string ("BinaryMagnitudeImageFilter"); }

      /** Print ourselves out */
      std::string ToString() const;


      /** Execute the filter on the input images */
#ifndef SWIG
      Image Execute ( Image&& image1, const Image& image2 );
#endif
      Image Execute ( const Image& image1, const Image& image2 );

    private:

      /** Setup for member function dispatching */

      using MemberFunctionType = Image (Self::*)( const Image& image1, const Image& image2 );
      template <class TImageType> Image ExecuteInternal ( const Image& image1, const Image& image2 );


      friend struct detail::MemberFunctionAddressor<MemberFunctionType>;

      std::unique_ptr<detail::MemberFunctionFactory<MemberFunctionType> > m_MemberFactory;



      bool m_InPlace{false};
    };

    /**\
     * \brief Computes the square root of the sum of squares of corresponding input pixels.
     *
     * This function directly calls the execute method of BinaryMagnitudeImageFilter
     * in order to support a procedural API
     *
     * \sa itk::simple::BinaryMagnitudeImageFilter for the object oriented interface
     * @{
     */
#ifndef SWIG
     SITKBasicFilters_EXPORT Image BinaryMagnitude ( Image&& image1, const Image& image2 );
#endif
     SITKBasicFilters_EXPORT Image BinaryMagnitude ( const Image& image1, const Image& image2 );

     /** @} */
}
#endif
